/*! concrete.css v2.1.1 | MIT License | github.com/louismerlin/concrete.css */

/**
 * Set universal color variables
 */

:root {
  --fg: #111;
  --bg: #fff;
}

/**
 * Invert the colors if the user has dark mode activated.
 */

@media (prefers-color-scheme: dark) {
  :root {
    --fg: #fff;
    --bg: #111;
  }
}

/**
 * 1. Modify the base font-size to 62.5% so that 1.6rem = 16px.
 * 2. Set box-sizing globally to handle padding and border widths.
 */

html {
  font-size: 62.5%; /* 1 */
  box-sizing: border-box; /* 2 */
}

/**
 * 1. Continue the global box-sizing modification.
 * 2. Set the underline size for all elements.
 */

*,
::after,
::before {
  box-sizing: inherit; /* 1 */
  text-decoration-thickness: 0.2rem; /* 2 */
}

/**
 * 1. Set global font size to 2rem (20px) with a normal weight.
 * 2. Set the text and background color to white and black by default.
 * 3. Set the font-family to Helvetica, or equivalent.
 */

body {
  font-size: 2rem; /* 1 */
  font-weight: 400; /* 1 */
  background: var(--bg); /* 2 */
  color: var(--fg); /* 2 */
  font-family: Helvetica, Arial, sans-serif; /* 3 */
}

/**
 * Set the link color to black
 */

a {
  color: var(--fg);
}

/**
 * 1. Set the max width of images to 100%, so that they don't overflow.
 * 2. Set the height of images relative to their width.
 */

img {
  max-width: 100%; /* 1 */
  height: auto; /* 2 */
}

/** 
 * 1. Center the main element horizontally.
 * 2. Set the width of the element to 640px, with padding for mobile screens.
 */

main {
  margin: auto; /* 1 */
  max-width: 66rem; /* 2 */
  padding: 0 1rem; /* 2 */
  width: 100%; /* 2 */
}

/**
 * 1. Add some padding around the header.
 * 2. Increase the size of text in the header.
 */

header {
  padding: 16rem 0; /* 1 */
  font-size: 1.2em; /* 2 */
}

/**
 * Center the text in the footer.
 */

footer {
  text-align: center;
}

/**
 * Add some padding around the sections.
 */

section {
  padding: 4rem 0;
}

/**
 * 1. Change the style of the button.
 * 2. Set the color and border of the button.
 * 3. Add the pointer cursor when hovering a button.
 */

button,
input[type="button"],
input[type="reset"],
input[type="submit"] {
  display: inline-block; /* 1 */
  vertical-align: middle; /* 1 */
  padding: 0.4rem 1rem; /* 1 */
  font-size: 2rem; /* 1 */
  font-weight: normal; /* 1 */
  margin-bottom: 1rem; /* 1 */
  background: var(--bg); /* 2 */
  color: var(--fg); /* 2 */
  border: 0.2rem solid var(--fg); /* 2 */
  border-radius: 0; /* 2 */
  cursor: pointer; /* 3 */
}

/**
 * 1. Set the border of a disabled button to be dashed.
 * 2. Add the not-allowed cursor when hovering a disabled button.
 */

button:disabled,
input[type="button"]:disabled,
input[type="reset"]:disabled,
input[type="submit"]:disabled {
  border-style: dashed; /* 1 */
  cursor: not-allowed; /* 2 */
}

/**
 * Set the list bullet to square.
 */

ul {
  list-style: square;
}

/**
 * Set the border around a fieldset.
 */

fieldset {
  border: 0.2rem solid var(--fg);
}

/**
 * 1. Modify the display of labels and legends.
 * 2. Add a margin underneath.
 */

label,
legend {
  display: block; /* 1 */
  font-weight: bold; /* 1 */
  margin-bottom: 0.8rem; /* 2 */
}

/** 
 * 1. Change the appearance of the text input.
 * 2. Set the color and border of the text input.
 */

input[type="email"],
input[type="number"],
input[type="password"],
input[type="search"],
input[type="tel"],
input[type="text"],
input[type="url"],
textarea,
select {
  -webkit-appearance: none; /* 1 */
  -moz-appearance: none; /* 1 */
  appearance: none; /* 1 */
  box-shadow: none; /* 1 */
  box-sizing: inherit; /* 1 */
  padding: 0.4rem 1rem; /* 1 */
  width: 100%; /* 1 */
  font-size: 2rem; /* 1 */
  color: var(--fg); /* 2 */
  background-color: var(--bg); /* 2 */
  border: 0.2rem solid var(--fg); /* 2 */
  border-radius: 0; /* 2 */
}

/**
 * Set margin for form elements.
 */

fieldset,
input,
select,
textarea {
  margin: 0 0 1.6rem 0;
}

/**
 * 1. Set the font color for the placeholder in inputs.
 * 2. Set font-style to italic for the placeholder in inputs.
 */

input::placeholder,
textarea::placeholder {
  color: var(--fg); /* 1 */
  font-style: italic; /* 2 */
}

/**
 * 1. Expand width to 100% for the table element.
 * 2. Remove the distance between the borders of adjacent table cells.
 */

table {
  width: 100%; /* 1 */
  border-spacing: 0; /* 2 */
}

/**
 * Add padding around table cells.
 */

td,
th {
  padding: 0.8rem;
}

/**
 * Remove left padding for first cell in a row.
 */

td:first-child,
th:first-child {
  padding-left: 0;
}

/**
 * Remove right padding for last cell in a row.
 */

td:last-child,
th:last-child {
  padding-right: 0;
}

/**
 * 1. Add a border under the table header.
 * 2. Align the text to the left in the table header.
 */

th {
  border-bottom: 0.2rem solid var(--fg); /* 1 */
  text-align: left; /* 2 */
}

/**
 * Add a thiner border under table rows.
 */

td {
  border-bottom: 0.1rem solid var(--fg);
}

/**
 * 1. Change the margins and padding.
 * 2. Add a border on the left of these elements.
 * 3. Set the y overflow to hidden to hide the navigation bar.
 */

blockquote,
pre {
  margin-left: 0; /* 1 */
  margin-right: 0; /* 1 */
  padding: 1rem 1.6rem; /* 1 */
  border-left: 0.2rem solid var(--fg); /* 2 */
  overflow-y: hidden; /* 3 */
}

/**
 * 1. Set the rest of the border of the preformatted element to dotted.
 * 2. Re-set the left border style.
 */

pre {
  border: 0.1rem dotted var(--fg);
  border-left: 0.2rem solid var(--fg);
}

/**
 * 1. Modify the display of the code block.
 * 2. Reduce the font size of the code block.
 */

pre > code {
  white-space: pre; /* 1 */
  display: block; /* 1 */
  font-size: 1.6rem; /* 2 */
}

/**
 * 1. Modify the display of the progress bar in all browsers.
 * 2. Set the colors and border of the progress bar.
 */

progress {
  -moz-appearance: none; /* 1 */
  -webkit-appearance: none; /* 1 */
  display: block; /* 1 */
  height: 1rem; /* 1 */
  overflow: hidden; /* 1 */
  padding: 0; /* 1 */
  width: 100%; /* 1 */
  background: var(--bg); /* 2 */
  color: var(--fg); /* 2 */
  border: 0.2rem solid var(--fg); /* 2 */
  border-radius: 0; /* 2 */
}

/**
 * Set the background color of the webkit progress bar.
 */

progress::-webkit-progress-bar {
  background-color: var(--bg);
}

/**
 * Set the color of the webkit progress bar.
 */

progress::-webkit-progress-value {
  background-color: var(--fg);
}

/**
 * Set the color of the mozilla progress bar.
 */

progress::-moz-progress-bar {
  background-color: var(--fg);
}

/**
 * Set the border for the horizontal rule.
 */

hr {
  border: 0.1rem solid var(--fg);
}
